/* Parallel Hierarchical Grid -- an adaptive finite element library.
 *
 * Copyright (C) 2005-2010 State Key Laboratory of Scientific and
 * Engineering Computing, Chinese Academy of Sciences. */

/* This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA */

/* $Id: partition-sfc.h,v 1.12 2009/06/29 12:53:50 liuhui Exp $ */
#ifndef PHG_PARTITION_SFC_H

/* Inverse Hilbert SFC */
/* for two-dimensional space, use co[0] and co[1] */
/* for three-dimensional spaces, use co[0], co[1] and co[2] */
typedef struct {
    SFC_FLOAT co[3];/* 0. <= x, y, z <= 1. */
    SFC_FLOAT sfc;  /* key */
    INT index;      /* local simplex index */
#if 0
    INT dim;        /* dimension */
#endif
}SFC;

/* scaling parameter */ 
static const FLOAT SFC_EPSILON = 1.6e-7;

#ifdef __cplusplus
extern "C" {
#endif

/* success: TRUE, fail: FALSE            */
/* n <= length of array hsfc             */
/* in general, n == length of array hsfc */
/* inverse Hilbert SFC                   */
BOOLEAN phgSFCInvHilbert3D(SFC *hsfc, INT n);
BOOLEAN phgSFCInvHilbert2D(SFC *hsfc, INT n);

/* inverse Morton SFC                   */
BOOLEAN phgSFCInvMorton3D(SFC *hsfc, INT n);
BOOLEAN phgSFCInvMorton2D(SFC *hsfc, INT n);

#if USE_MPI
/* Hilbert space-filling curve partitioner */
BOOLEAN phgPartitionSFC(GRID *g, int nprocs, DOF *weights, FLOAT power);
#endif

#ifdef __cplusplus
}
#endif

#define PHG_PARTITION_SFC_H
#endif
